"""
This module provides a database engine connection for the substrate database. It reads the username, password, host,
and port from AWS Parameter Store for better security.
This connection provides READ ONLY access to the Substrate database.
"""
from sqlalchemy import create_engine
from .utils import get_parameters


USER_PARAMETER = 'db-connector-substrate-username'
PASSWORD_PARAMETER = 'db-connector-substrate-password'
HOST_PARAMETER = 'db-connector-substrate-host'
PORT_PARAMETER = 'db-connector-substrate-port'

DB_NAME = 'substrate'

parameter_to_value_map = get_parameters([USER_PARAMETER, PASSWORD_PARAMETER, HOST_PARAMETER, PORT_PARAMETER])

user = parameter_to_value_map[USER_PARAMETER]
password = parameter_to_value_map[PASSWORD_PARAMETER]
host = parameter_to_value_map[HOST_PARAMETER]
port = parameter_to_value_map[PORT_PARAMETER]

engine = create_engine(
    f"postgresql://{user}:{password}@{host}:{port}/{DB_NAME}",
    connect_args={"options": "-c statement_timeout=60000"},
)



"""
This module provides a database engine connection for the SquadIQ database. It reads the username, password, and host
from AWS Parameter Store for better security.
This connection provides READ ONLY access to the SquadIQ database.
"""
from sqlalchemy import create_engine
from .utils import get_parameters

AWS_REGION = "ap-south-1"

JUPYTERHUB_PG_DB_NAME_PARAMETER = "squadiq-jupyterhub-pg-db-name"
JUPYTERHUB_PG_USERNAME_PARAMETER = "squadiq-jupyterhub-pg-username"
JUPYTERHUB_PG_PASSWORD_PARAMETER = "squadiq-jupyterhub-pg-password"
JUPYTERHUB_PG_HOST_PARAMETER = "squadiq-replica-pg-host"

parameter_to_value_map = get_parameters([
    JUPYTERHUB_PG_USERNAME_PARAMETER,
    JUPYTERHUB_PG_PASSWORD_PARAMETER,
    JUPYTERHUB_PG_HOST_PARAMETER,
    JUPYTERHUB_PG_DB_NAME_PARAMETER,
], AWS_REGION)

user = parameter_to_value_map[JUPYTERHUB_PG_USERNAME_PARAMETER]
password = parameter_to_value_map[JUPYTERHUB_PG_PASSWORD_PARAMETER]
host = parameter_to_value_map[JUPYTERHUB_PG_HOST_PARAMETER]
db_name = parameter_to_value_map[JUPYTERHUB_PG_DB_NAME_PARAMETER]

engine = create_engine(
    "postgresql://{user}:{password}@{host}/{db_name}".format(user=user, password=password, host=host, db_name=db_name),
    connect_args={"options": "-c statement_timeout=60000"},
)



"""
This module provides a database engine connection for the bifrost database. It reads the username, password, and host
from AWS Parameter Store for better security.
This connection provides READ ONLY access to the Bifrost database.
"""
from sqlalchemy import create_engine
from .utils import get_parameters


USER_PARAMETER = 'db-connector-bifrost-username'
PASSWORD_PARAMETER = 'db-connector-bifrost-password'
HOST_PARAMETER = 'db-connector-bifrost-host'

parameter_to_value_map = get_parameters([USER_PARAMETER, PASSWORD_PARAMETER, HOST_PARAMETER])

user = parameter_to_value_map[USER_PARAMETER]
password = parameter_to_value_map[PASSWORD_PARAMETER]
host = parameter_to_value_map[HOST_PARAMETER]

engine = create_engine(
    f"postgresql://{user}:{password}@{host}/bifrost",
    connect_args={"options": "-c statement_timeout=60000"},
)



import boto3

from collections import namedtuple


ssm_client = boto3.client('ssm', region_name='us-east-1')


def get_parameter(parameter_name: str) -> str:
    """
    Fetches the value of the given parameter from AWS SSM Parameter Store.
    :param parameter_name: String
    :return: String
    """
    return ssm_client.get_parameter(
        Name=parameter_name, WithDecryption=True
    )['Parameter']['Value']


def get_parameters(parameter_names: [str], region='us-east-1') -> {str: str}:
    """
    Fetches the given parameters from SSM Parameter store.
    :param parameter_names: List of strings
    :param region: String
    :return: dictionary containing key-value pairs
    """
    ssm_client = boto3.client('ssm', region_name=region)

    parameters = ssm_client.get_parameters(
        Names=parameter_names, WithDecryption=True
    )['Parameters']

    parameter_to_value_map = {
        parameter['Name']: parameter['Value']
        for parameter in parameters
    }

    return parameter_to_value_map


def dict_fetch_all(cursor):
    """
    Return all rows from a cursor as a dict.
    By default, the Python DB API will return results without their field names, which means you end up with a
    list of values, rather than a dict. At a small performance and memory cost, we can return results as a
    dict.

    Using Normal Cursor -
    >>> cursor.execute("SELECT id, parent_id FROM test LIMIT 2");
    >>> cursor.fetchall()
    ((54360982, None), (54360880, None))

    Using dict_fetch_all method -
    >>> cursor.execute("SELECT id, parent_id FROM test LIMIT 2");
    >>> dictfetchall(cursor)
    [{'parent_id': None, 'id': 54360982}, {'parent_id': None, 'id': 54360880}]
    """
    columns = [col[0] for col in cursor.description]
    return [
        dict(zip(columns, row))
        for row in cursor.fetchall()
    ]


def namedtuple_fetch_all(cursor):
    """
    Return all rows from a cursor as a namedtuple.
    A namedtuple is a tuple-like object that has fields accessible by attribute lookup;
    it’s also indexable and iterable. Results are immutable and accessible by field names or indices,
    which might be useful.

    Using Normal Cursor -
    >>> cursor.execute("SELECT id, parent_id FROM test LIMIT 2");
    >>> cursor.fetchall()
    ((54360982, None), (54360880, None))

    Using namedtuple_fetch_all method -
    >>> cursor.execute("SELECT id, parent_id FROM test LIMIT 2");
    >>> results = namedtuplefetchall(cursor)
    >>> results
    [Result(id=54360982, parent_id=None), Result(id=54360880, parent_id=None)]
    >>> results[0].id
    54360982
    >>> results[0][0]
    54360982
    """

    desc = cursor.description
    nt_result = namedtuple('Result', [col[0] for col in desc])
    return [nt_result(*row) for row in cursor.fetchall()]



"""
This module provides a database engine connection for the decision_science database. It reads the from host
from AWS Parameter Store for better security. And reads username and password for a user from their home
directory in the JupyterHub server.
This connection provides WRITE access to resources inside a user's own schema in the decision_science database.
And provides READ access to the whole database.
"""
import csv
import getpass

from sqlalchemy import create_engine
from .utils import get_parameter


HOST_PARAMETER = 'db-connector-decision-science-host'

host = get_parameter(HOST_PARAMETER)

# read username and password from the credentials file from user's directory
ubuntu_user = getpass.getuser()

with open(f'/home/{ubuntu_user}/decision_science_creds.csv', 'r') as f:
    reader = csv.DictReader(f)

    for row in reader:
        username = row['username']
        password = row['password']

engine = create_engine(
    f"postgresql://{username}:{password}@{host}/decision_science",
    connect_args={"options": "-c statement_timeout=60000"},
)



import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...

install_requires = [
    'boto3==1.14.60', 'psycopg2-binary==2.8.6', 'SQLAlchemy==1.3.19'
]

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="squad-sagemaker-db-connector",
    version="0.2",
    author="Ujjwal Gupta",
    author_email="ujjwal.gupta@squadstack.com",
    description=("A pluggable connector that allows users (admins) to execute SQL,"
                 " view, and export the results."),
    license="MIT",
    keywords="Db connector",
    url="https://github.com/squadrun/auctm-database-connector",
    packages=['database_connector'],
    long_description=read('README.rst'),
    long_description_content_type='text/plain',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Utilities',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ],
    install_requires=install_requires,
    include_package_data=True,
    zip_safe=False,
)
